// -*- C++ -*-
// $RCSfile: arraybase.C,v $
// $Revision: 1.2 $
// $Author: reida $
// $Date: 2003/04/02 14:51:34 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

// Base class for Array templates, and other non-templated Array stuff

#include "array.h"
#include "coord.h"
#include "geometry.h"

void ArrayBase::resize(const ICoord &size) {
  if(height == size(1) && width == size(0)) return;
  height = size(1);
  width = size(0);
  free();
  allocate();
}

ArrayIterator::ArrayIterator(const ICRectangle &r, const ARI_Type flag)
  : i(r.ymin()), j(r.xmin()),
    x0(r.xmin()), y0(r.ymin()),
    x1(r.xmax()), y1(r.ymax())
{
  if(flag == ARI_INCLUSIVE) { // limits are inclusive
    x1++;
    y1++;
  }
}

int ArrayIterator::operator()(ICoord &pxl) {
  if(j == x1) {
    j = x0;
    i++;
    if(i >= y1) return 0;	// ">" is required for 0x0 array!
  }
  pxl(0) = j;
  pxl(1) = i;
  j++;
  return 1;
}

void ArrayIterator::reset() {
  i = y0;
  j = x0;
}

int ArrayBase::contains(const ICoord &pxl) const {
  if(pxl(0) < 0 || pxl(0) >= width) return 0;
  if(pxl(1) < 0 || pxl(1) >= height) return 0;
  return 1;
}

