// -*- C++ -*-
// $RCSfile: lock.h,v $
// $Revision: 1.10 $
// $Author: langer $
// $Date: 2006/01/18 22:24:06 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef LOCK_H
#define LOCK_H


#include <sys/types.h>
#include <pthread.h>

// Simple wrapper for a mutex lock.
class Lock {
private:
  pthread_mutex_t lock;
public:
  Lock();
  ~Lock();
  void acquire();
  void release();
};


// Wrapper for the custom RWLock.  Having a custom class means not
// having to cope with arhitecture-dependent implementations, or the
// even-more-inconvenient absence thereof.
class RWLock {
private:
  int r, w, p;			// read, write, pause
  pthread_mutex_t local_lock;
  // Condition signalled when r=w=0 becomes true.
  pthread_cond_t rw_zero;
public:
  RWLock();
  ~RWLock();
  void write_acquire();
  void write_release();

  void read_acquire();
  void read_release();

  void write_pause();
  void write_resume();

  int nReaders() const { return r; }
};


// The KeyHolder is a wrapper for a mutex lock.  The constructor
// acquires the lock and the destructor releases it.  Creating a
// KeyHolder object in a function allows the lock to be held
// throughout the scope of the function, that is, until just before
// the function returns.

class KeyHolder {
private:
  Lock  *lock;
public:
  KeyHolder(Lock &other_lock);
  ~KeyHolder();
};

// enable_all and disable_all override all instances of Lock and
// RWLock.  They're used to prevent deadlocks when running in
// unthreaded mode.

void enable_all();
void disable_all();

#endif // LOCK_H
