// -*- C++ -*-
// $RCSfile: ooferror.C,v $
// $Revision: 1.18 $
// $Author: langer $
// $Date: 2006/02/15 16:47:29 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include <iostream>
#include "common/ooferror.h"
#include "common/swiglib.h"
#include "common/threadstate.h"
#include "common/tostring.h"
#include "common/trace.h"



std::string ErrBadIndex::message() const {
  return ErrProgrammingErrorBase<ErrBadIndex>::message() +
    "Bad Index: " + to_string(badindex);
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// Python equivalents 

// Makes the most general assumption about the structure.
std::string ErrProgrammingError::pythonequiv() const {
  return "ErrProgrammingError('" + msg + "', '" + file + "', " +
    to_string(line) + ")";
}

std::string ErrResourceShortage::pythonequiv() const {
  return "ErrResourceShortage('" + msg + "')";
}

std::string ErrBadIndex::pythonequiv() const {
  return "ErrBadIndex('" + ErrProgrammingErrorBase<ErrBadIndex>::message() +
    "')";
}

std::string ErrUserError::pythonequiv() const {
  return "ErrUserError('" + msg + "')";
}

std::string ErrSetupError::pythonequiv() const {
  return "ErrSetupError('" + msg + "')";
}

std::string ErrProcessAborted::pythonequiv() const {
  return "ErrProcessAborted('" + pname + "')";
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// Function to convert a Python exception into a C++ exception.  This
// in effect is the inverse of the swig exception typemap.  The two of
// them together allow an exception to be batted back and forth
// between C++ and Python until somebody decides to catch it.

// pythonErrorRelay() should be called whenever C++ detects that a
// Python exception has been raised in a Python API call.  It converts
// the Python exception to a C++ exception if possible (ie, when the
// Python exceptioon is really a swigged C++ ErrError subclass).
// Otherwise it just throws PythonError, which is caught by the swig
// exception typemap.

void pythonErrorRelay() {
  acquirePyLock();
  try {
    PyObject *ptype;
    PyObject *pvalue;
    PyObject *ptraceback;
    PyErr_Fetch(&ptype, &pvalue, &ptraceback); // clears the Python error state.
    if(ptype) {
      // Get the swigged ErrError class, so that we can compare error
      // types on the Python side.
      static PyObject *pyErrError = 0;
      if(!pyErrError) {
	PyObject *module = PyImport_ImportModule("oof2.SWIG.common.ooferror");
	pyErrError = PyObject_GetAttrString(module, "ErrErrorPtr");
	Py_XDECREF(module);
      }
      if(PyErr_GivenExceptionMatches(ptype, pyErrError)) {
	// The Python exception is a swigged ErrError -- ie, it's one of
	// ours.  Extract the C++ object and raise it as a new
	// exception.
	const ErrError *ee;
	SWIG_GetPtrObj(pvalue, (void**) &ee, "_ErrError_p");
	ee->throw_self();
      }
      else {
	// The error is a native Python exception.  Since PyErr_Fetch
	// cleared the error state, we have to call PyErr_Restore.
	// The error will be caught again when C++ returns to Python,
	// at which point the swig exception typemap will handle it.
	PyErr_Restore(ptype, pvalue, ptraceback);
	throw PythonError();
      }
    }
  }
  catch (...) {
    releasePyLock();
    throw;
  }
  releasePyLock();
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

// This isn't the real initialization for pyconverter, this is just
// here to make the linker happy.  The real initialization happens in
// the function below at import time.
PyObject *ErrError::pyconverter(0);

// Initializes the static pyconverter member of the ErrError base
// class -- called from the SWIG'd Python file at ooferror import
// time.  Errors which occur prior to this import will not be handled
// correctly.
void pyErrorInit(PyObject *converter) {
  Py_XINCREF(converter);
  ErrError::pyconverter = converter;
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

void testerror() {
  Trace("testerror");
  throw ErrProgrammingError("This is just a test.", __FILE__, __LINE__);
}
