# -*- python -*-
# $RCSfile: analyzemenu.py,v $
# $Revision: 1.25 $
# $Author: reida $
# $Date: 2006/01/31 21:33:16 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 



from oof2.SWIG.common import switchboard
from oof2.common import enum
from oof2.common import utils
from oof2.common.IO import automatic
from oof2.common.IO import destinationparam
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO import output
from oof2.common.IO import parameter
from oof2.common.IO import reporter
from oof2.common.IO import whoville
from oof2.common.IO import xmlmenudump
from oof2.engine.IO import analyze
from oof2.engine import analysisdomain
from oof2.engine import analysissample
import oof2.engine.mesh
import sys

ops_menu = oofmenu.OOFMenuItem(
    "Analyze",
    secret=1,
    help="Compute properties of the solution.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/menu/analyze.xml'))

mainmenu.OOF.Mesh.addItem(ops_menu)

# Parameters to all the menu items.

mesh_param =  whoville.WhoParameter(
    'mesh', oof2.engine.mesh.meshes,
    tip="The mesh on which to compute the output")

output_param = output.ValueOutputParameter(
    'data', tip="The output data source")

domain_param = parameter.RegisteredParameter(
    'domain', analysisdomain.Domain,
    tip="Where on the mesh to compute the data.")

sample_param = parameter.RegisteredParameter(
    'sampling', analysissample.SampleSet,
    tip="How to sample the domain.")

destination_param = destinationparam.DestinationParameter(
    "destination", value=automatic.automatic, default=automatic.automatic,
    tip="File where data should be written, or 'automatic' for the message window.") 

formatting_params =  [
    parameter.StringParameter("comment_character",
                              "#", default="#",
                              tip="Set the string at the start of the header line."),
    enum.EnumParameter("separator", analyze.Separator,
                       "comma", default="comma",
                       tip="Set the character between data entries.")]


common_analysis_params = [
    mesh_param, output_param, domain_param,
    sample_param, destination_param ] + formatting_params


def _ops_callback(menuitem, mesh, data, domain, sampling, destination,
                  comment_character, separator, **kwargs):
    callable = menuitem.data()

    # "destination" is a file name, or "automatic" -- use
    # destinationparam.get_file to convert it to a file handle.
    outfile = destinationparam.get_file(destination)
    # Then set the mesh in the domain, and run the operation.
    domain.set_mesh(mesh)

    domain.read_lock()
    try:
        # The domain has all the mesh data, so we don't have to pass that data.
        callable(data, domain, sampling, outfile,
                 comment_character, separator)
        outfile.flush()
    finally:
        domain.read_release()

# Create a menu item for each registered object in the DataOperation
# class.
def makemenu(menu_base):
    menu_base.clearSubMenu()
    for r in analyze.DataOperation.registry:
        try:
            help = r.tip
        except KeyError:
            help = None
        new_item = oofmenu.OOFMenuItem(
            utils.space2underscore(r.name()),
            callback = _ops_callback,
            params = common_analysis_params + r.params,
            help=help,
            discussion=r.discussion)
        new_item.data = r
        menu_base.addItem(new_item)

makemenu(ops_menu)

switchboard.requestCallback(analyze.DataOperation, makemenu, ops_menu)
