// -*- C++ -*-
// $RCSfile: timestepadapter.h,v $
// $Revision: 1.9 $
// $Author: langer $
// $Date: 2005/09/18 18:56:18 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#ifndef TIMESTEPADAPTER_H
#define TIMESTEPADAPTER_H

class SteppingScheme;
class SolverDriverCore;
class FEMesh;


class TimeStepAdapter {
private:
  double tolerance;
protected:
  double dt; // suggested dt
  bool continew; //flag to stop/continue the computation. default is true
  VECTOR_D * x_prev;/*
		      Solution from the previous step.
		      Will be restored if step adapter fails
		    */
public:
  TimeStepAdapter() : tolerance(0), dt(0), continew(true), x_prev(0) {}
  TimeStepAdapter(const double tol) : tolerance(tol), dt(0), continew(true), x_prev(0) {}
  virtual ~TimeStepAdapter();
  void set_tolerance(const double teef) {tolerance = teef;}
  double get_tolerance() {return tolerance;}
  double get_dt() {return dt;}
  virtual bool is_okay(SteppingScheme *smethod) =0;
  
  
  //hooks for subclasses
  virtual void accept(FEMesh *femesh, SteppingScheme *smethod); //solution is accepted and time index is updated
  virtual void reject(FEMesh *femesh); //solution is rejected. time index is NOT updated
  virtual void set_time_step(FEMesh * femesh, SteppingScheme *smethod) = 0;
  virtual double get_residual(SteppingScheme *smethod) = 0; //each subclass knows how to set the residual
  virtual bool keep_going() = 0;
};



class NullAdapter : public TimeStepAdapter {
public:
  NullAdapter() : TimeStepAdapter() {}
  virtual ~NullAdapter() {}
  
  virtual void set_time_step(FEMesh * femesh, SteppingScheme *smethod);
  virtual double get_residual(SteppingScheme *smethod); //returns zero
  virtual void accept(FEMesh *femesh, SteppingScheme *smethod);//solution is accepted and time index is updated
  virtual void reject(FEMesh *femesh);//solution is rejected. time index is NOT updated
  virtual bool keep_going() {return continew;}
  virtual bool is_okay(SteppingScheme *smethod);
};


#endif // TIMESTEPADAPTER_H
